@@ -17,15 +17,45 @@
  *  Netspeed Applet was writen by Jörgen Scheibengruber <mfcn@gmx.de>
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#if defined(sun) && defined(__SVR4)
+#include <sys/sockio.h>
+#endif
+
 #include <glibtop/netlist.h>
 #include <glibtop/netload.h>
+
+#ifdef HAVE_IW
+ #include <iwlib.h>
+#endif /* HAVE_IW */
+
 #include "backend.h"
 
 gboolean
-is_no_dummy_device(const char* device)
+is_dummy_device(const char* device)
 {
-	return !g_str_has_prefix(device, "lo") && !g_str_has_prefix(device, "dummy");
+	glibtop_netload netload;
+	glibtop_get_netload(&netload, device);
+
+	if (netload.if_flags & (1 << GLIBTOP_IF_FLAGS_LOOPBACK))
+		return TRUE;
+
+	/* Skip interfaces without any IPv4/IPv6 address (or
+	   those with only a LINK ipv6 addr) However we need to
+	   be able to exclude these while still keeping the
+	   value so when they get online (with NetworkManager
+	   for example) we don't get a suddent peak.  Once we're
+	   able to get this, ignoring down interfaces will be
+	   possible too.  */
+	if (!(netload.flags & (1 << GLIBTOP_NETLOAD_ADDRESS6)
+	      && netload.scope6 != GLIBTOP_IF_IN6_SCOPE_LINK)
+	    && !(netload.flags & (1 << GLIBTOP_NETLOAD_ADDRESS)))
+		return TRUE;
+
+	return FALSE;
 }
 
 
@@ -73,7 +103,7 @@ get_default_route(void)
 		
 		if (retval != 11) continue;
 			
-		if (gw == 0 && is_no_dummy_device(device)) {
+		if (gw == 0 && !is_dummy_device(device)) {
 			fclose(fp);
 			return device;
 		}			
@@ -102,6 +132,10 @@ free_device_info(DevInfo *devinfo)
 	g_free(devinfo->ptpip);
 	g_free(devinfo->hwaddr);
 	g_free(devinfo->ipv6);
+	g_free(devinfo->essid);
+	g_free(devinfo->tx_rate);
+	g_free(devinfo->rx_rate);
+	g_free(devinfo->sum_rate);
 }
 
 
@@ -174,6 +208,8 @@ get_device_info(const char *device, DevInfo *devinfo)
 	devinfo->ip = format_ipv4(netload.address);
 	devinfo->netmask = format_ipv4(netload.subnet);
 	devinfo->ipv6 = format_ipv6(netload.address6);
+	devinfo->qual = 0;
+	devinfo->essid = NULL;
 
 	hw = netload.hwaddress;
 	if (hw[6] || hw[7]) {
@@ -192,9 +228,16 @@ get_device_info(const char *device, DevInfo *devinfo)
 	if(netload.if_flags & (1L << GLIBTOP_IF_FLAGS_LOOPBACK)) {
 		devinfo->type = DEV_LO;
 	}
+
+#ifdef HAVE_IW
+
 	else if (netload.if_flags & (1L << GLIBTOP_IF_FLAGS_WIRELESS)) {
 		devinfo->type = DEV_WIRELESS;
+		get_wireless_info (devinfo);
 	}
+
+#endif /* HAVE_IW */
+
 	else if(netload.if_flags & (1L << GLIBTOP_IF_FLAGS_POINTOPOINT)) {
 		if (g_str_has_prefix(device, "plip")) {
 			devinfo->type = DEV_PLIP;
@@ -231,3 +274,43 @@ compare_device_info(const DevInfo *a, const DevInfo *b)
 
 	return FALSE;	
 }
+#ifdef HAVE_IW
+void
+get_wireless_info (DevInfo *devinfo)
+{
+	int fd;
+	wireless_info info = {0};
+
+	fd = iw_sockets_open ();
+
+	if (fd < 0)
+		return;
+
+	if (iw_get_basic_config (fd, devinfo->name, &info.b) < 0) 
+		goto out;
+	
+	if (info.b.has_essid) {
+		if ((!devinfo->essid) || (strcmp (devinfo->essid, info.b.essid) != 0)) {
+			devinfo->essid = g_strdup (info.b.essid);
+		}
+	} else {
+		devinfo->essid = NULL;
+	}
+
+	if (iw_get_stats (fd, devinfo->name, &info.stats, &info.range, info.has_range) >= 0)
+		info.has_stats = 1;
+
+	if (info.has_stats) {
+		if (devinfo->qual != info.stats.qual.qual) {
+			devinfo->qual = info.stats.qual.qual;
+		}
+	} else {
+		devinfo->qual = 0;
+	}
+
+	goto out;
+out:
+	if (fd != -1)
+		close (fd);
+}
+#endif /* HAVE_IW */
